@@ -3,7 +3,6 @@ package enterpriselinux
 import (
 	"strings"
 
-	"github.com/k0sproject/k0sctl/configurer"
 	k0slinux "github.com/k0sproject/k0sctl/configurer/linux"
 	"github.com/k0sproject/rig"
 	"github.com/k0sproject/rig/os/registry"
@@ -20,9 +19,7 @@ func init() {
 			return os.ID == "rhel" && !strings.Contains(os.Name, "CoreOS")
 		},
 		func() interface{} {
-			linuxType := &RHEL{}
-			linuxType.PathFuncs = interface{}(linuxType).(configurer.PathFuncs)
-			return linuxType
+			return &RHEL{}
 		},
 	)
 }
